judpApp.factory("Util", function ($rootScope, $http, APP_CONSTANTS, $cordovaCapture,
                                  $q, $filter, $ionicPopup, $httpParamSerializer, $cordovaNetwork, $cordovaFile,
                                  $ionicLoading, $stateParams, $ionicModal, $state, $cordovaFileOpener2, $window,
                                  $cordovaFileTransfer, $timeout) {
    var Util = {
        jpush: {
            init: function (receiveNotification, openNotification) {
                if (window.plugins && window.plugins.jPushPlugin) {
                    var isProduct = Setting.environment == "production";
                    window.plugins.jPushPlugin.init();
                    window.plugins.jPushPlugin.setDebugMode(!isProduct);
                    window.plugins.jPushPlugin.setDebugMode(false);
                    window.plugins.jPushPlugin.setStatisticsOpen(isProduct);

                    //document.addEventListener("jpush.receiveNotification", receiveNotification, false);
                    //document.addEventListener("jpush.openNotification", openNotification, false);
                }
            },
            setAlias: function (alias) {
                if (window.plugins && window.plugins.jPushPlugin)
                    window.plugins.jPushPlugin.setAlias(alias);
            }
        },
        common: {
            getNumber: function (value) {
                var num = +(((value || "").toString().match(/^\d+(\.\d+)?/g) || [])[0]);
                if (isNaN(num)) {
                    num = 0;
                }
                return num;
            },
            calculateRate: function (column, total) {
                if (total == 0) {
                    return 100;
                } else {
                    return (column / total * 100 ).toFixed(2);
                }
            },
            debug: function () {
                if (APP_CONSTANTS.DEBUG) {
                    console.log.apply(window.console, arguments);
                }
            },
            info: function () {
                if (APP_CONSTANTS.DEBUG) {
                    console.info.apply(window.console, arguments);
                }
            },
            error: function () {
                if (APP_CONSTANTS.DEBUG) {
                    console.error.apply(window.console, arguments);
                }
            },
            warn: function () {
                if (APP_CONSTANTS.DEBUG) {
                    console.warn.apply(window.console, arguments);
                }
            },
            i18n: function (key) {
                return $filter("translate")(key);
            },
            i18nFormat: function (key) {
                return $filter("translateFormat").apply("", arguments);
            },
            alert: function (str, title) {
                title = title || APP_CONSTANTS.APP_NAME;
                navigator.notification.alert(str, function () {
                }, title, "确定");
            },
            confirm: function (str, config, cb) {
                if (typeof config == 'function') {
                    cb = config;
                    config = {};
                }
                if (!navigator.notification)
                    return cb("1");

                var title = config['title'] || APP_CONSTANTS.APP_NAME;
                navigator.notification.confirm(str, cb, title, [config["okButtonText"] 
                    || "确定", config["cancelButtonText"] 
                    || "取消"]);
            },
            toast: function (msg) {
                if (deviceUtil.desktop()) {
                    alert(msg);
                    return;
                }
                window.plugins.toast.showWithOptions({
                    message: msg,
                    duration: "short",
                    position: "top",
                    styling: {
                        cornerRadius: 12,
                        horizontalPadding: 20,
                        verticalPadding: 12,
                        opacity: 1
                    }
                });
            },
            toastFormat: function (i18nKey) {
                window.plugins.toast.showWithOptions({
                    message: $filter("translateFormat").apply("", arguments),
                    duration: "short",
                    position: "top",
                    styling: {
                        cornerRadius: 12,
                        horizontalPadding: 20,
                        verticalPadding: 12,
                        opacity: 1
                    }
                });
            },
            dlg: function (msg, title) {
                title = title || PATROL_CONSTANTS.APP_NAME;
                $ionicPopup.alert({
                    title: title,
                    template: msg
                });
            },
            stringFormat: function () {
                if (arguments.length == 0)
                    return "";
                var str = arguments[0];
                for (var i = 1; i < arguments.length; i++) {
                    var re = new RegExp('\\{' + (i - 1) + '\\}', 'gm');
                    str = str.replace(re, arguments[i]);
                }
                return str;
            },
            showLoading: function (msg) {
                var template = "<ion-spinner icon='ios'></ion-spinner>";
                if (msg) {
                    template += "<div>" + msg + "</div>";
                }
                $ionicLoading.show({
                    template: template
                });
            },
            hideLoading: function () {
                $ionicLoading.hide();
            },
            guid: function guid() {
                return 'xxxxxxxxxxxx4xxxyxxxxxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                    var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
                    return v.toString(16);
                });
            },
            barcodeScan: function(params){
                var q = $q.defer();
                
                if (!params)
                  params = {
                      text_title: "", // Android only
                      text_instructions: "", // Android only
                      camera: "back" ,// defaults to "back"
                      flash: "auto" ,// defaults to "auto". See Quirks
                      drawSight: true //defaults to true, create a red sight/line in the center of the scanner view.
                  };

                  cloudSky.zBar.scan(params, function (result) {
                      q.resolve(result);
                  }, function (err) {
                      q.reject(err);
                  });

                  return q.promise;
            },
            isSuccess: function(data){
                return angular.isObject(data) && data.success && angular.isObject(data.data);
            }
        },
        chartHelper: {
            renderPie: function (selector, data) {
                var chart = echarts.init(selector);
                var option = {
                    //backgroundColor: "#147eea",
                    calculable: false,
                    series: [
                        {
                            name: '访问来源',
                            type: 'pie',
                            radius: '40%',
                            center: ['50%', '50%'],
                            data: data,
                            itemStyle: {
                                normal: {
                                    label: {
                                        formatter: function (params) {
                                            return (params.name || "").replace(/(.{9})/ig, "$1\n") + ":" + params.percent + "%";
                                        }
                                    }
                                }
                            }
                        }
                    ]
                };
                chart.setOption(option);
                return chart;
            },
            renderBar: function (selector, data) {
                var chart = echarts.init(selector);
                var option = {
                    grid: {
                        top: "4%",
                        left: '3%',
                        right: '4%',
                        bottom: '3%',
                        containLabel: true
                    },
                    yAxis: [
                        {
                            type: 'category',
                            data: data.keys,
                            axisTick: {
                                show: false
                            },
                            axisLabel: {
                                interval: 0
                            }
                        }
                    ],
                    xAxis: [
                        {
                            type: 'value'
                        }
                    ],
                    series: [
                        {
                            name: '直接访问',
                            type: 'bar',
                            label: {
                                normal: {
                                    show: true,
                                    position: "right",
                                    formatter: '{c}'
                                }
                            },
                            itemStyle: {
                                normal: {
                                    color: function (param) {
                                        return "#" + Util.common.guid().slice(0, 6);
                                    }
                                }
                            },
                            data: data.values
                        }
                    ]
                };
                chart.setOption(option);
                return chart;
            }
        },
        networkHelper: {
            is3GAtleast: function () {
                if (!deviceUtil.desktop()) {
                    var state = $cordovaNetwork.getNetwork();
                    return state == Connection.CELL_3G || state == Connection.CELL_4G || state == Connection.WIFI;
                } else
                    return true;
            },
            isWifi: function () {
                if (!deviceUtil.desktop()) {
                    var state = $cordovaNetwork.getNetwork();
                    return state == Connection.WIFI;
                } else
                    return true;
            },
            is3G: function () {
                var state = $cordovaNetwork.getNetwork();
                return state == Connection.CELL_3G;
            },
            isOnline: function () {
                if (!deviceUtil.desktop()) {
                    return $cordovaNetwork.isOnline();
                } else
                    return true;
            }
        },
        ajax: {
            isNetAvailable: function (net) {
                return true;
                net = (net || "").toLowerCase();
                if (net == "all") {
                    return Util.networkHelper.is3GAtleast();
                } else if (net == "wifi") {
                    return Util.networkHelper.isWifi();
                }
            },
            isNotLogin: function (response) {
                return response.message.indexOf("not login") != -1 || response.message == "login failure";
            },
            get: function(url, params, options) {
                var q = $q.defer();
    
                if (!Util.ajax.isNetAvailable("all")) {
                    Util.common.toast("网络连接错误！");
                    Util.common.hideLoading();
                    q.reject(APP_CONSTANTS.NET.NETUNCONNECT);
                } else if (!url) {
                    Util.common.hideLoading();
                    q.reject(APP_CONSTANTS.NET.UNKNOWNREMOTEURL)
                } else {
                    params = params || {};
                    options = options || {};
                    url = APP_CONSTANTS.SERVER + url;
                    var config = {
                        method: "get",
                        timeout: options.timeout || 20000
                    };

                    if (url.indexOf("?") > -1) {
                        url += "&" + $httpParamSerializer(params);
                    } else {
                        url += "?" + $httpParamSerializer(params);
                    }
                    
                    $http.get(url, config).then(function (data) {
                        var response = data.data || {};
                        Util.common.warn(Util.common.stringFormat("get $http to {0} success:", url), params);
                        Util.common.warn(response);
                        Util.common.hideLoading();

                        if (angular.isObject(response)) {
                            q.resolve(response);
                        } else {
                            if (angular.isString(response) && response.indexOf("<html>") != -1) {
                                $state.go("login", {}, {reload: true});
                                q.reject();
                            } else {
                                Util.common.alert(Util.common.stringFormat("\n错误信息:{0}\n请求路径:{1}\n请截图反馈此错误信息", response, url), "请求失败");
                                q.reject(response);
                            }
                        }
                    }, function (error) {
                        Util.common.hideLoading();
                        Util.common.error(Util.common.stringFormat("get $http to {0} error:", url), params, error);
                        q.reject(error);
                    });
                }
                return q.promise;
            },
            post: function(url, params, options){
                var q = $q.defer();
                if (!Util.ajax.isNetAvailable("all")) {
                    Util.common.toast("网络连接错误！");
                    Util.common.hideLoading();
                    q.reject(APP_CONSTANTS.NET.NETUNCONNECT);
                } else if (!url) {
                    Util.common.hideLoading();
                    q.reject(APP_CONSTANTS.NET.UNKNOWNREMOTEURL)
                } else {
                    params = params || {};
                    options = options || {};
                    url = APP_CONSTANTS.SERVER + url;
                    var config = {
                        method: "post",
                        timeout: options.timeout || 20000
                    };

                    var params = JSON.stringify(params);
                    $http.post(url, params, config).then(function (data) {
                        var response = data.data || {};
                        Util.common.warn(Util.common.stringFormat("post $http to {0} success:", url), params);
                        Util.common.warn(response);
                        Util.common.hideLoading();

                        if (angular.isObject(response)) {
                            q.resolve(response);
                        } else {
                            if (angular.isString(response) && response.indexOf("<html>") != -1) {
                                $state.go("login", {}, {reload: true});
                                q.reject();
                            } else {
                                Util.common.alert(Util.common.stringFormat("\n错误信息:{0}\n请求路径:{1}\n请截图反馈此错误信息", response, url), "请求失败");
                                q.reject(response);
                            }
                        }
                    }, function (error) {
                        Util.common.hideLoading();
                        Util.common.error(Util.common.stringFormat("post $http to {0} error:", url), params, error);
                        q.reject(error);
                    });
                }
                return q.promise;
            },
            remote: function (options) {
                var q = $q.defer();
    
                if (!Util.syncHelper.isNetAvailable("all")) {
                    Util.common.toast("网络连接错误！");
                    Util.common.showloadhideLoading();
                    q.reject(APP_CONSTANTS.NET.NETUNCONNECT);
                } else if (!options.url) {
                    Util.common.hideLoading();
                    q.reject(APP_CONSTANTS.NET.UNKNOWNREMOTEURL)
                } else {
                    options.data = options.data || {};
                    options.type = options.type || "get";
                    options.url = APP_CONSTANTS.SERVER + options.url;
                    var config = {
                        method: options.type,
                        url: options.url,
                        timeout: options.timeout || 20000
                    };

                    if (options.url.indexOf("?") > -1) {
                        options.url += "&token=" + APP_CONSTANTS.token;
                    } else {
                        options.url += "?token=" + APP_CONSTANTS.token
                    }

                    if (options.type == "get") {
                        options.url += "&" + $httpParamSerializer(options.data);

                        $http.get(options.url, config).then(function (data) {
                                var response = data.data || {};
                                Util.common.info(Util.common.stringFormat("get $http from {0} success:", options.url));
                                Util.common.info(response);
                                if (response.success || response.success === undefined || options.ignoreError) {
                                    q.resolve(angular.copy(response));
                                } else {
                                    if (response.message.indexOf("<html>") != -1) {
                                        if (isLoginPage) {
                                            q.resolve(angular.copy(response));
                                        } else {
                                            $state.go("login");
                                            Util.common.toast("请您重新登录！");
                                        }
                                    } else {
                                        Util.common.alert(Util.common.stringFormat("错误信息:{0}\n请求路径:{1}\n请截图反馈此错误信息", response.message, options.url), "请求失败");
                                        q.reject(response);
                                    }
                                    Util.common.hideLoading();
                                }
                            }, function (error) {
                                Util.common.hideLoading();
                                Util.common.error(Util.common.stringFormat("get $http from {0} error:", options.url), error);
                                q.reject(error);
                            }
                        );
                    } else if (options.type == "post") {
                        $http.post(options.url, options.data, config).then(function (data) {
                            var response = data.data || {};
                            Util.common.warn(Util.common.stringFormat("post $http to {0} success:", options.url), options.data);
                            Util.common.warn(response);
                            if (response.success) {
                                q.resolve(angular.copy(response));
                            } else {
                                if (response.message.indexOf("<html>") != -1) {
                                    $state.go("login", {}, {reload: true});
                                } else {
                                    Util.common.alert(Util.common.stringFormat("\n错误信息:{0}\n请求路径:{1}\n请截图反馈此错误信息", response.message, options.url), "请求失败");
                                    q.reject(response);
                                }
                                Util.common.hideLoading();
                            }
                            q.resolve(data.data);
                        }, function (error) {
                            Util.common.hideLoading();
                            Util.common.error(Util.common.stringFormat("post $http to {0} error:", options.url), options.data, error);
                            q.reject(error);
                        })
                    }
                }
                return q.promise;
            }
        },
        versionHelper: {
            checkAppUpdate: function (version) {
                if (ionic.Platform.isAndroid()) {
                    this.checkAndroid(version);
                } else {
                    this.checkIos(version);
                }
            },
            checkAndroid: function (version) {
                this.check(APP_CONSTANTS.DEPLOYINFO.ANDROID_SHORTCUT, APP_CONSTANTS.DEPLOYINFO.API_KEY, version);
            },
            checkIos: function (version) {
                this.check(APP_CONSTANTS.DEPLOYINFO.IOS_SHORTCUT, APP_CONSTANTS.DEPLOYINFO.API_KEY, version);
            },
            downloadapp: function (appKey, apiKey) {
                $ionicLoading.show({
                    template: "已经下载：0%"
                });
                var url = "http://www.pgyer.com/apiv1/app/install?aKey=" + appKey + "&_api_key=" + apiKey;
                var targetPath = cordova.file.externalApplicationStorageDirectory +  "/booksExchangeApp.ipa";
                var trustHosts = true;
                var options = {};
                $cordovaFileTransfer.download(url, targetPath, options, trustHosts).then(function (result) {
                        // 打开下载下来的APP
                        $cordovaFileOpener2.open(targetPath, 'application/vnd.android.package-archive'
                    ).then(function () {
                        // 成功
                    }, function (err) {
                        // 错误
                        window.open("https://www.pgyer.com/txjsd", '_system');
                    });
                    $ionicLoading.hide();
                }, function (err) {
                    $ionicLoading.show({template: '下载失败！', noBackdrop: true, duration: 2000});
                    window.open("https://www.pgyer.com/txjsd", '_system');
                }, function (progress) {
                    //进度，这里使用文字显示下载百分比
                    $timeout(function () {
                        var downloadProgress = (progress.loaded / progress.total) * 100;
                        $ionicLoading.show({
                            template: "已经下载：" + Math.floor(downloadProgress) + "%"
                        });
                        if (downloadProgress >= 100) {
                            $ionicLoading.hide();
                        }
                    })
                });
            },

            check: function (shortcut, _api_key, version) {
                var postCfg = {
                    headers: {'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'}
                };
                var self = this;
                $http.post("http://www.pgyer.com/apiv1/app/getAppKeyByShortcut", "shortcut=" + shortcut + "&_api_key=" + _api_key, postCfg).then(function (result) {
                    var appResult = result.data.data;
                    if (appResult.appKey && version !== appResult.appVersion) {
                        $http.post("http://www.pgyer.com/apiv1/app/view", "aKey=" + appResult.appKey + "&_api_key=" + _api_key, postCfg).then(function (result) {
                            var versionResult = result.data.data;
                            $ionicPopup.confirm({
                                title: versionResult.appVersion + '版本更新内容',
                                template: versionResult.appUpdateDescription.replace(/\n+/g, "<br>"),
                                cancelText: '取消',
                                okText: '更新'
                            }).then(
                                function (res) {
                                    if (res) {
                                        if (!appResult.appKey) {
                                            alert('无法获取App Key,可以点击下方的版本按钮手动更新');
                                        } else {
                                            //Util.common.showLoading("正在更新,如果自动更新失败,可以点击下方的版本按钮手动更新");
                                            Util.common.showLoading("正在更新......");
                                            $timeout(function () {
                                                Util.common.hideLoading();
                                            }, 3000);
                                            if (ionic.Platform.isAndroid()) {
                                                self.downloadapp(appResult.appKey, _api_key);
                                            } else {
                                                //window.open("http://www.pgyer.com/apiv1/app/install?aKey=" + appResult.appKey + "&_api_key=" + _api_key, '_self');
                                                window.open('http://www.pgyer.com/' + APP_CONSTANTS.DEPLOYINFO.IOS_SHORTCUT, '_system');
                                            }
                                        }
                                    }
                                });
                        })
                    }
                }, function () {

                });
            }
        },
        fileHelper: {
            getRealPath: function (path) {
                var q = $q.defer();
                if (ionic.Platform.isAndroid()) {
                    window.FilePath.resolveNativePath(path, function (fileNativeUrl) {
                        q.resolve(fileNativeUrl);
                    })
                } else {
                    window.resolveLocalFileSystemURL(path, function (file) {
                        q.resolve(file.nativeURL);
                    });
                }
                return q.promise;
            },
            saveMedia: function (media) {
                var q = $q.defer();
                var ext = media.fileName.substring(media.fileName.lastIndexOf('.') + 1);
                var sourcePath = media.filePath;
                var targetPath = Util.common.stringFormat("{0}/tmp/medias/{1}.{2}", Util.fileHelper.getDataDir(), Util.common.guid(), ext);
                if (sourcePath.indexOf("file:") != 0) {
                    sourcePath = "file://" + sourcePath;
                }
                $cordovaFileTransfer.download(sourcePath, targetPath).then(function (file) {
                    media.filePath = file.nativeURL;
                    if (media.mediaType == APP_CONSTANTS.MEDIA.TYPE.VIDEO) {
                        window.plugins.videoThumbnail.build(function (thumbPath) {
                            media.thumbnailPath = thumbPath;
                            q.resolve(media);
                        }, function (error) {
                            q.reject(error);
                        }, {
                            videoPath: file.nativeURL,
                            width: 200,
                            height: 200,
                            kind: 1
                        });
                    } else {
                        q.resolve(media);
                    }
                }, function () {
                    console.log(arguments)
                });
                return q.promise;
            },
            removeMedia: function (media) {
                if (media.status == APP_CONSTANTS.MEDIA.STATUS.FILEINAPP) {
                    var fileDir = media.src.match(/^.+(?=\/[^/]+?$)/)[0];
                    var fileName = media.src.match(/[^/]+?$/)[0];
                    $cordovaFile.removeFile(fileDir, fileName).then(function () {
                        if (media.type == APP_CONSTANTS.MEDIA.TYPE.AUDIO) {
                            var pngName = fileName.slice(0, -4) + ".png";
                            return $cordovaFile.removeFile(fileDir, pngName);
                        } else
                            return true;
                    });
                }
            },
            getDataDir: function (isExternalData) {
                var destDir = "";
                if (ionic.Platform.isIOS()) {
                    destDir = cordova.file.documentsDirectory;
                } else {
                    if (isExternalData)
                        destDir = cordova.file.externalDataDirectory;
                    else
                        destDir = cordova.file.applicationStorageDirectory + "databases/";
                }

                return destDir;
            },
            getLocalFileFromMedia: function (media, isThumbnail) {
                media = angular.copy(media);
                var q = $q.defer();
                if (media.fileId) {
                    var sourcePath = isThumbnail ? media.src + "&viewImage=true" : media.src;
                    var targetPath = Util.common.stringFormat("{0}tmp/medias/{1}{2}.jpg", Util.fileHelper.getDataDir(), media.fileId, isThumbnail ? "_thumbnail" : "");
                    $cordovaFileTransfer.download(sourcePath, targetPath).then(function (fileEntry) {
                        var img = new Image();
                        img.onload = function () {
                            q.resolve(targetPath, {});
                        };
                        img.onerror = function () {
                            q.resolve("", {});
                        };
                        img.src = targetPath;
                    }, function () {
                        q.resolve(sourcePath, {});
                    });
                } else {
                    q.resolve(media.src);
                }
                return q.promise;
            },
            resizeImage: function (path) {
                var q = $q.defer();
                Util.fileHelper.getRealPath(path).then(function (realPath) {
                    var image = new Image();
                    image.onload = function () {
                        window.ImageResizer.resize({
                            uri: realPath,
                            folderName: "capture",
                            quality: 75,
                            width: this.width / 2,
                            height: this.height / 2
                        }, function (imageUrl) {
                            q.resolve(imageUrl);
                        }, function () {
                            q.resolve(path);
                        });
                    };
                    image.onerror = function () {
                        q.resolve(path)
                    };
                    image.src = path;
                });
                return q.promise;
            }
        },
        mediaHelper: {
            capturePhoto: function () {
                var q = $q.defer();
                $cordovaCapture.captureImage().then(function (imageData) {
                    Util.fileHelper.saveMedia({
                        fileSize: imageData[0].size,
                        fileName: imageData[0].name,
                        filePath: imageData[0].fullPath,
                        mediaType: APP_CONSTANTS.MEDIA.TYPE.IMAGE
                    }).then(function (data) {
                        Util.common.debug(JSON.stringify(data));
                        q.resolve(data);
                    }, function (error) {
                        q.reject(error);
                    });
                }, function (error) {
                    q.reject(error);
                });
                return q.promise;
            },
            captureVideo: function () {
                var q = $q.defer();
                navigator.device.capture.captureVideo(function (imageData) {
                    Util.fileHelper.saveMedia({
                        fileSize: imageData[0].size,
                        fileName: imageData[0].name,
                        filePath: imageData[0].fullPath,
                        mediaType: APP_CONSTANTS.MEDIA.TYPE.VIDEO
                    }).then(function (data) {
                        q.resolve(data);
                    }, function (error) {
                        q.reject(error);
                    });
                }, function (error) {
                    q.reject(error);
                });
                return q.promise;
            },
            captureAudio: function (savedFilePath) {
                var q = $q.defer();
                var fileName = savedFilePath.split('/')[savedFilePath.split('/').length - 1];
                $window.resolveLocalFileSystemURL(cordova.file.dataDirectory,
                    function (fileSystem) {
                        fileSystem.getFile(fileName, {create: false}, function (fileEntry) {
                            fileEntry.file(function (audioData) {
                                Util.fileHelper.saveMedia({
                                    fileSize: audioData.size,
                                    fileName: fileName,
                                    filePath: savedFilePath,
                                    mediaType: APP_CONSTANTS.MEDIA.TYPE.AUDIO
                                }).then(function (data) {
                                    q.resolve(data);
                                }, function (error) {
                                    q.reject(error);
                                });
                            });
                        }, function (error) {
                            q.reject(error);
                        });
                    }, function (error) {
                        q.reject(error);
                    });

                return q.promise;
            },
            getLocalPhoto: function () {
                var q = $q.defer();
                navigator.camera.getPicture(function (imageData) {
                    Util.fileHelper.getRealPath(imageData).then(function (path) {
                        q.resolve(path);
                    });
                }, function (error) {
                    q.reject(error)
                }, {
                    destinationType: Camera.DestinationType.FILE_URI,
                    sourceType: Camera.PictureSourceType.PHOTOLIBRARY
                });
                return q.promise;
            }
        }
    };
    return Util;
});
